var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
import { defineComponent, ref, onMounted, createVNode, markRaw, watch, unref, nextTick, getCurrentInstance, onBeforeUnmount, withDirectives, resolveComponent, resolveDirective } from "vue";
import "codemirror/lib/codemirror.css";
import _CodeMirror from "codemirror";
import "codemirror/addon/merge/merge.css";
import "codemirror/addon/merge/merge.js";
import DiffMatchPatch from "diff-match-patch";
import "codemirror/addon/mode/simple.js";
const CodeMirror$2 = window.CodeMirror || _CodeMirror;
var Default = defineComponent({
  name: "defaultMode",
  props: {
    name: {
      type: String,
      default: `cm-textarea-${+new Date()}`
    },
    value: {
      type: String,
      default: ""
    },
    content: {
      type: String,
      default: ""
    },
    options: {
      type: Object,
      default: () => ({})
    },
    cminstance: {
      type: Object,
      default: () => null
    },
    placeholder: {
      type: String,
      default: ""
    }
  },
  emits: {
    ready: (instance) => instance,
    "update:cminstance": (instance) => instance
  },
  setup(props, {
    emit
  }) {
    const textarea = ref();
    const _cminstance = ref(null);
    const initialize = () => {
      _cminstance.value = markRaw(CodeMirror$2.fromTextArea(textarea.value, props.options));
      emit("update:cminstance", _cminstance.value);
      let unwatch = watch(() => props.cminstance, (val, oldVal) => {
        var _a;
        val && ((_a = props.cminstance) == null ? void 0 : _a.setValue(props.value || props.content));
        emit("ready", unref(_cminstance));
        unwatch == null ? void 0 : unwatch();
      }, {
        deep: true
      });
    };
    onMounted(() => {
      initialize();
    });
    return () => createVNode("textarea", {
      "ref": "textarea",
      "name": "{props.name}",
      "placeholder": "{props.placeholder}"
    }, null);
  }
});
const CodeMirror$1 = (window == null ? void 0 : window.CodeMirror) || _CodeMirror;
window.diff_match_patch = DiffMatchPatch;
window.DIFF_DELETE = -1;
window.DIFF_INSERT = 1;
window.DIFF_EQUAL = 0;
var Merge = defineComponent({
  name: "mergeMode",
  props: {
    options: {
      type: Object,
      default: () => ({})
    },
    cminstance: {
      type: Object,
      default: () => ({})
    }
  },
  emits: ["update:cminstance", "ready"],
  setup(props, {
    emit
  }) {
    const _cminstance = ref();
    const mergeView = ref();
    const initialize = () => {
      _cminstance.value = markRaw(CodeMirror$1.MergeView(mergeView.value, props.options));
      emit("update:cminstance", _cminstance.value);
      emit("ready", _cminstance);
    };
    onMounted(() => {
      initialize();
    });
    return () => createVNode("div", {
      "ref": "mergeView"
    }, null);
  }
});
var errorType;
(function(errorType2) {
  errorType2["info"] = "info";
  errorType2["warning"] = "warning";
  errorType2["error"] = "error";
})(errorType || (errorType = {}));
function getLocalTime() {
  const date = new Date();
  const h = date.getHours() < 10 ? "0" + date.getHours() : date.getHours();
  const m = date.getMinutes() < 10 ? "0" + date.getMinutes() : date.getMinutes();
  const s = date.getSeconds() < 10 ? "0" + date.getSeconds() : date.getSeconds();
  return `${h}:${m}:${s}`;
}
function createLinkMark(attrs) {
  return `#link#${JSON.stringify(attrs)}#link#`;
}
function getLinkMark(value) {
  const linkRegexp = /#link#(.+)#link#/g;
  let result = [];
  let indexObj;
  indexObj = linkRegexp.exec(value);
  while (indexObj) {
    const node = document.createElement("a");
    const attrs = JSON.parse(indexObj[1]);
    const keyAndValues = Object.entries(attrs);
    for (let [_key, _value] of keyAndValues) {
      node.setAttribute(_key, _value);
    }
    node.className = "editor_custom_link";
    node.innerHTML = "logDownload";
    result.push({
      start: indexObj.index,
      end: indexObj.index + indexObj[0].length,
      node
    });
    indexObj = linkRegexp.exec(value);
  }
  return result;
}
function createLogMark(text = "", type = "info") {
  return `#log<${type}>log#${text}#log<${type}>log#`;
}
function getLogMark(value) {
  let result = [];
  function match() {
    const logRegexp = new RegExp(`#log<(\\w*)>log#((.|\r
|
)*?)#log<(\\w*)>log#`, "g");
    let indexObj;
    indexObj = logRegexp.exec(value);
    while (indexObj) {
      let text = indexObj[0].replace(/\r\n/g, "\n");
      let textArr = text.split("\n");
      let content = indexObj[2].replace(/\r\n/g, "\n");
      let contentArr = content.split("\n");
      const node = document.createElement("span");
      let type = indexObj[1];
      node.className = `c-editor--log__${type}`;
      let offset = 0;
      for (let i = 0; i < textArr.length; i++) {
        let textItem = textArr[i];
        let contentItem = contentArr[i];
        let cloneNode = node.cloneNode(false);
        cloneNode.innerText = contentItem;
        result.push({
          start: indexObj.index + offset,
          end: indexObj.index + offset + textItem.length,
          node: cloneNode
        });
        offset = offset + textItem.length + 1;
      }
      indexObj = logRegexp.exec(value);
    }
  }
  match();
  return result;
}
function createLog(log, type = "") {
  let now = getLocalTime();
  return `[${now}] <${type}> ${log}`;
}
function createTitle(title = "", baseLength = 20, symbol = "=") {
  const offsetLength = Math.floor(1.5 * title.length / 2);
  let arr = new Array(Math.max(baseLength - offsetLength, 5));
  const wraptext = arr.join(symbol);
  return `${wraptext}${title}${wraptext}`;
}
const startRegex = [
  {
    regex: /(\[.*?\])([ \t]*)(<error>[ \t])(.+)/,
    token: ["tag", "", "error.strong", "error.strong"],
    sol: true
  },
  {
    regex: /(\[.*?\])([ \t]*)(<info>)(.+)(.?)/,
    token: ["tag", "", "bracket", "bracket", "hr"],
    sol: true
  },
  {
    regex: /(\[.*?\])([ \t]*)(<warning>)(.+)(.?)/,
    token: ["tag", "", "comment", "comment", "hr"],
    sol: true
  }
];
_CodeMirror.defineSimpleMode("fclog", {
  start: [
    ...startRegex,
    {
      regex: /.*/,
      token: "hr"
    }
  ],
  error: [
    ...startRegex,
    {
      regex: /.*/,
      token: "error.strong"
    }
  ],
  info: [
    ...startRegex,
    {
      regex: /.*/,
      token: "bracket"
    }
  ],
  warning: [
    ...startRegex,
    {
      regex: /.*\[/,
      token: "comment"
    }
  ]
});
_CodeMirror.defineSimpleMode("log", {
  start: [
    {
      regex: /^[=]+[^=]*[=]+/,
      token: "strong"
    },
    {
      regex: /([^\w])([A-Z][\w]*)/,
      token: ["", "string"]
    },
    {
      regex: /(^[A-Z][\w]*)/,
      token: "string"
    }
  ]
});
const CodeMirror = window.CodeMirror || _CodeMirror;
var FcLog = defineComponent({
  name: "codemirror-fclog",
  props: {
    value: {
      type: String,
      default: ""
    },
    name: {
      type: String,
      default: `cm-textarea-${+new Date()}`
    },
    options: {
      type: Object,
      default: () => ({})
    },
    cminstance: {
      type: Object,
      default: () => ({})
    },
    placeholder: {
      type: String,
      default: ""
    }
  },
  emits: ["update:cminstance", "ready"],
  setup(props, {
    emit
  }) {
    const textarea = ref();
    const _cminstance = ref(null);
    const renderTextMark = (cminstance = props.cminstance) => {
      const marks = cminstance.getAllMarks();
      marks.forEach((mark) => mark.clear());
      const value = cminstance.getValue();
      const linkMarks = [].concat(getLinkMark(value)).concat(getLogMark(value));
      for (let _i = 0; _i < linkMarks.length; _i++) {
        const mark = linkMarks[_i];
        cminstance.markText(cminstance.posFromIndex(mark.start), cminstance.posFromIndex(mark.end), {
          replacedWith: mark.node
        });
      }
    };
    const initialize = () => {
      var _a;
      _cminstance.value = markRaw(CodeMirror.fromTextArea(textarea.value, props.options));
      emit("update:cminstance", unref(_cminstance));
      (_a = _cminstance.value) == null ? void 0 : _a.on("change", renderTextMark);
    };
    watch(() => props.cminstance, (val) => {
      var _a;
      if (val) {
        renderTextMark(props.cminstance);
        (_a = props.cminstance) == null ? void 0 : _a.setValue(props.value);
        emit("ready", _cminstance);
      }
    }, {
      deep: true,
      immediate: true
    });
    onMounted(() => {
      initialize();
    });
    return () => createVNode("textarea", {
      "ref": "textarea",
      "name": props.name,
      "placeholder": props.placeholder
    }, null);
  }
});
const componentsEvts = {
  "update:value": (value) => value,
  change: (value, cm) => null,
  input: (value) => "",
  ready: (cm) => cm
};
const DEFAULT_OPTIONS = {
  mode: "text",
  theme: "default",
  lineNumbers: true,
  smartIndent: true,
  indentUnit: 2,
  foldGutter: true,
  matchBrackets: true,
  autoCloseBrackets: true,
  styleActiveLine: true
};
function useViewControl({
  props,
  cminstance,
  presetRef
}) {
  const containerWidth = ref(null);
  const containerHeight = ref(null);
  const refresh = () => {
    nextTick(() => {
      cminstance.value.refresh();
    });
  };
  const resize = (width = props.width, height = props.height) => {
    containerWidth.value = String(width).replace("px", "");
    containerHeight.value = String(height).replace("px", "");
    let cmHeight = containerHeight.value;
    cminstance.value.setSize(containerWidth.value, cmHeight);
  };
  const destroy = () => {
    const element = cminstance.value.doc.cm.getWrapperElement();
    element == null ? void 0 : element.remove();
  };
  const isStyleChaotic = () => {
    const gutterEl = document.querySelector(".CodeMirror-gutters");
    const gutterElLeft = gutterEl == null ? void 0 : gutterEl.style.left.replace("px", "");
    return gutterElLeft != "0";
  };
  const reviseStyle = () => {
    refresh();
    if (!isStyleChaotic())
      return;
    let timer = setInterval(() => {
      isStyleChaotic() ? refresh() : clearInterval(timer);
    }, 60);
    let clearTimer = setTimeout(() => {
      clearInterval(timer);
      clearTimeout(clearTimer);
    }, 400);
  };
  return {
    refresh,
    resize,
    destroy,
    containerHeight,
    reviseStyle
  };
}
if (typeof Object.assign != "function") {
  Object.defineProperty(Object, "assign", {
    value(target, varArgs) {
      if (target == null) {
        throw new TypeError("Cannot convert undefined or null to object");
      }
      const to = Object(target);
      for (let index2 = 1; index2 < arguments.length; index2++) {
        const nextSource = arguments[index2];
        if (nextSource != null) {
          for (const nextKey in nextSource) {
            if (Object.prototype.hasOwnProperty.call(nextSource, nextKey)) {
              to[nextKey] = nextSource[nextKey];
            }
          }
        }
      }
      return to;
    },
    writable: true,
    configurable: true
  });
}
var Codemirror = defineComponent({
  name: "CodemirrorEditor",
  props: {
    value: {
      type: String,
      default: ""
    },
    marker: {
      type: Function,
      default: () => null
    },
    unseenLines: Array,
    name: {
      type: String,
      default: ""
    },
    placeholder: {
      type: String,
      default: ""
    },
    merge: {
      type: Boolean,
      default: false
    },
    options: {
      type: Object,
      default: () => DEFAULT_OPTIONS
    },
    globalOptions: {
      type: Object,
      default: () => DEFAULT_OPTIONS
    },
    border: {
      type: Boolean,
      default: false
    },
    width: {
      type: [String, Number],
      default: null
    },
    height: {
      type: [String, Number],
      default: null
    },
    KeepCursorInEnd: {
      type: Boolean,
      default: false
    }
  },
  emits: __spreadValues({}, componentsEvts),
  components: {
    Default,
    Merge,
    FcLog
  },
  setup(props, {
    emit,
    attrs
  }) {
    const cminstance = ref(null);
    const content = ref("");
    const presetModeName = ref("Default");
    const cmOptions = ref(Object.assign(__spreadValues({}, DEFAULT_OPTIONS), props.globalOptions, props.options));
    const internalInstance = getCurrentInstance();
    const presetRef = ref(null);
    const {
      refresh,
      resize,
      destroy,
      containerHeight,
      reviseStyle
    } = useViewControl({
      props,
      cminstance,
      presetRef
    });
    const unseenLineMarkers = () => {
      if (props.unseenLines !== void 0 && props.marker !== void 0) {
        props.unseenLines.forEach((line) => {
          var _a, _b;
          const info = (_a = cminstance.value) == null ? void 0 : _a.lineInfo(line);
          (_b = cminstance.value) == null ? void 0 : _b.setGutterMarker(line, "breakpoints", (info == null ? void 0 : info.gutterMarkers) ? null : props.marker());
        });
      }
    };
    const onCodeChange = (newVal) => {
      var _a, _b;
      const cm_value = (_a = cminstance.value) == null ? void 0 : _a.getValue();
      if (newVal !== cm_value) {
        (_b = cminstance.value) == null ? void 0 : _b.setValue(newVal);
        content.value = newVal;
        reviseStyle();
      }
      unseenLineMarkers();
    };
    const handlePresetModeName = () => {
      if (props.options.mode == "fclog" || props.options.mode == "log") {
        presetModeName.value = "FcLog";
        return;
      }
      if (props.merge) {
        presetModeName.value = "Merge";
        return;
      }
      presetModeName.value = "Default";
    };
    watch(() => props.options, (val) => {
      var _a;
      for (const key in props.options) {
        (_a = cminstance.value) == null ? void 0 : _a.setOption(key, val[key]);
      }
    }, {
      deep: true
    });
    watch(() => props.value, (val) => {
      onCodeChange(val);
    });
    watch(() => props.merge, (val) => {
      handlePresetModeName();
    }, {
      immediate: true
    });
    onBeforeUnmount(() => {
      destroy();
    });
    return () => {
      var _a, _b;
      return createVNode("div", {
        "class": ["codemirror-container", props.merge && "merge", (props.border || props.merge) && "bordered", (!props.width || props.width == "100%") && "width-auto", (!props.height || props.height == "100%") && "height-auto"],
        "style": {
          height: containerHeight + "px"
        }
      }, [withDirectives(createVNode(resolveComponent("component"), {
        "style": "height: 100%",
        "is": presetModeName,
        "ref": "presetRef",
        "cminstance": cminstance,
        "onUpdate:cminstance": ($event) => cminstance = $event,
        "options": cmOptions,
        "name": props.name || ((_b = (_a = internalInstance == null ? void 0 : internalInstance.parent) == null ? void 0 : _a.type) == null ? void 0 : _b.name) || void 0,
        "content": content
      }, null), [[resolveDirective("on-ready"), "ready"], [resolveDirective("bind"), props, attrs]])]);
    };
  }
});
var index = "";
Codemirror.install = (app, config) => {
  if (config) {
    if (config.options) {
      Codemirror.props.globalOptions.default = () => config.options;
    }
    if (config.events) {
      Codemirror.props.globalEvents.default = () => config.events;
    }
  }
  app.component("Codemirror", Codemirror);
  return app;
};
function styleInject(css,ref){if(ref===void 0){ref={}}var insertAt=ref.insertAt;if(!css||typeof document==="undefined"){return}var head=document.head||document.getElementsByTagName("head")[0];var style=document.createElement("style");style.type="text/css";if(insertAt==="top"){if(head.firstChild){head.insertBefore(style,head.firstChild)}else{head.appendChild(style)}}else{head.appendChild(style)}if(style.styleSheet){style.styleSheet.cssText=css}else{style.appendChild(document.createTextNode(css))}};styleInject(``);
export { Codemirror, createLinkMark, createLog, createLogMark, createTitle, Codemirror as default, getLinkMark, getLocalTime, getLogMark };
